#!/usr/bin/perl -w

################################################################################
# CONFIGURATION
################################################################################

# a deplacer dans un fichier de conf "ipphones.conf" dans le repertoire courant

my $dir = "/srv/www/ipphones";
my $firmware_re = '^ipphone5-(\d{3})-auf(\d{1,2}).bin';
my $base_conf = "base.cfg";
my $utilisateurs = "utilisateurs";

my $debug=0;

################################################################################
use strict;
sub current_firmware();
sub configuration_version($);
sub version_file($);
sub version_extension($);
sub extension_utilisateur($);
sub texte_a_envoyer($);
sub fichier_a_envoyer($);
sub envoyer_texte();

my $req = $ENV{QUERY_STRING};

my $texte_retour = "";

my ($type,$file) = split(/=/,$req);

die "type : $type" unless($type =~ m/^(set|cfg)$/);
die "file : $file" unless($file =~ m/^[a-z][.-a-z]*/);

print "$type/$file\n" if($debug);

chdir($dir);

if($type eq "set")
{
my $firmware = current_firmware();
my $configuration = configuration_version($file);

texte_a_envoyer(<<EOT);
BIN $firmware
CFG $file.cfg:$configuration
END
EOT
}
elsif($type eq "cfg")
{
    texte_a_envoyer("[SETTING]\n");

    # balancer le fichier general, s'il existe
    if(version_file("$base_conf"))
    {
	texte_a_envoyer("##\n## FICHIER : $base_conf\n##\n");
	fichier_a_envoyer("$base_conf") 
    }

    # quelle extension pour cette personne ?
    if(my $extension = extension_utilisateur($file))
    {
	texte_a_envoyer("phonenumber = $extension\n");
    }
    # balancer le fichier specifique, s'il existe
    if(version_file("$file.cfg"))
    {
	texte_a_envoyer("##\n## FICHIER : $file.cfg\n##\n");
	fichier_a_envoyer("$file.cfg") ;
    }
    texte_a_envoyer("[END]\n");
}

envoyer_texte();

exit 0;


################################################################################
#                           FONCTIONS UTILITAIRES
################################################################################

sub version_firmware($)
{
    my ($nom_firmware) = @_;
    my $retour = '00000';
    if($nom_firmware =~ /$firmware_re/)
    {
	$retour = sprintf("%d%02d",$1,$2);
    }   
    return ($retour);
}

# trouver le firmware le plus recent
sub current_firmware()
{
    my $retour="erreur";
    open LS,"ls|" or die "ls: $!";
    my @firmware = sort { version_firmware($a) cmp version_firmware($b);  } (grep {/$firmware_re/} <LS>);
    close LS;
    if(@firmware && $firmware[$#firmware] =~ /$firmware_re/)
    {
	my $nom_firmware = $firmware[$#firmware];
	chomp $nom_firmware;
	$retour = "$nom_firmware:".version_firmware($nom_firmware);
    }
    return  $retour;
}

# retourne le numero de serie d'un fichier de conf
# zero si le fichier n'existe pas
sub version_file($)
{
    my ($file) = @_;
    my $retour=0;

    print "FILE:$file\n" if($debug);

    if(open FILE,$file)
    {
	while(<FILE>)
	{
	    $retour = $1 if(/#\s*SERIE\s*:\s*(\d+)/);
	}
        close FILE;
    }
    return $retour;
}

# retourne le numero de serie associe a une extension
# 0 si pas de numero de version
sub version_extension($)
{
    my ($utilisateur) = @_;

    my $version = 0;
    if(open(FILE,"$utilisateurs"))
    {
	while(<FILE>)
	{
	    $version = $1 if(/^$utilisateur\s+\d+\s+(\d+)/);
	}
	close FILE;
    }
    return $version;
}

# Calculer la version associee a un utilisateur
sub configuration_version($)
{
    my $retour = 0;
    my ($user) = @_;

    $retour = version_file($base_conf) + version_file("$user.cfg");
    $retour += version_extension($user);
}


# retourne l'extension associee a un utilisateurs, si le fichier $utilisateurs existe bien
# et zero si pas d'info disponible
sub extension_utilisateur($)
{
    my ($utilisateur) = @_;
    my $extension = 0;
    if(open(FILE,"$utilisateurs"))
    {
	while(<FILE>)
	{
	    $extension = $1 if(/^$utilisateur\s+(\d+)/);
	}
	close FILE;
    }
    return $extension;
}

################################################################################
# CGI !!

# a ajouter au texte de reponse a la requete
sub texte_a_envoyer($)
{
    $texte_retour .= $_[0];
}

sub fichier_a_envoyer($)
{
    my ($fichier) = @_;
    open FILE,$fichier or die "$fichier : $!";
    map {texte_a_envoyer($_)} (grep {! /^(#.*|\s*)$/} <FILE>);
    close FILE;
}

# reponse effective a la requete : balancer tout ce qui a ete accumule via a_envoyer()
sub envoyer_texte()
{
    my $longueur = length($texte_retour);
    print "Content-Length: $longueur\nContent-Type: text/plain\n\n";
    print $texte_retour;
}
